-- ChunkyChunks. Factorio mod: Show grid lines.
-- Copyright (C) 2016  Qon

-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>

local mod_name = 'ChunkyChunks'

local SHORTCUT_NAME = 'chunkychunks-toggle'

local debugging = false

function chunky(player_index, size, offset, color)
    local p = game.players[player_index]
    if color == nil then
        color = p.color
        color.a = 1
    end
    local pos = p.position
    local chunk = size
    local r = math.max(150, chunk*2)
    local width = math.ceil(chunk/16)
    local offset = {x=offset, y=offset}
    pos = {x=pos.x - offset.x, y=pos.y - offset.y}
    local topleft = {x=math.ceil((pos.x-r)/chunk)*chunk, y=math.ceil((pos.y-r)/chunk)*chunk}
    topleft = {x=topleft.x + offset.x, y=topleft.y + offset.y}
    local bottomright = {x=math.floor((pos.x+r)/chunk)*chunk, y=math.floor((pos.y+r)/chunk)*chunk}
    bottomright = {x=bottomright.x + offset.x, y=bottomright.y + offset.y}

    -- p.print(game.table_to_json({x=math.floor(pos.x), y=math.floor(pos.y)}))
    pos = {x=math.floor(pos.x/chunk)*chunk, y=math.floor(pos.y/chunk)*chunk}
    local l = 65
    for x = topleft.x, bottomright.x, chunk do
        rendering.draw_line{
          color=color, width=width,
          --[[gap_length=…, dash_length=…,--]]
          from={x, topleft.y},
          to={x, bottomright.y},
          surface=p.surface,
          players={p},
          draw_on_ground=true,
        }
    end
    for y = topleft.y, bottomright.y, chunk do
        rendering.draw_line{
          color=color, width=width,
          --[[gap_length=…, dash_length=…,--]]
          from={topleft.x, y},
          to={bottomright.x, y},
          surface=p.surface,
          players={p},
          draw_on_ground=true,
        }
    end
end

function enabled(player_index, set)
    local p = game.players[player_index]
    if set ~= nil then p.set_shortcut_toggled(SHORTCUT_NAME, set) end
    return p.is_shortcut_toggled(SHORTCUT_NAME)
end

function toggle(event)
    -- game.print(game.table_to_json(event))
    if event.input_name ~= SHORTCUT_NAME and event.prototype_name ~=  SHORTCUT_NAME then return end
    enabled(event.player_index, not enabled(event.player_index))
    -- local p = game.players[event.player_index]
    -- printOrFly(p, enabled(event.player_index))
end

function printOrFly(p, text)
    if p.character ~= nil then
        p.create_local_flying_text({
            ['text'] = text,
            ['position'] = p.character.position,
            -- time_to_live = 2,
        })
    else
        p.print(text)
    end
end

function string:split(sep)
   local sep, fields = sep or ":", {}
   local pattern = string.format("([^%s]+)", sep)
   self:gsub(pattern, function(c) fields[#fields+1] = c end)
   return fields
end

function parseColour(s)
    -- local a = {}
    -- for v in string.gmatch(s, "%S+") do
    --     table.insert(a, v)
    -- end
    local a = s:split(' ')
    if #a < 3 or #a > 4 then return nil end
    for i, v in ipairs(a) do
        v = string.match(v, "%d%.?%d*")
        if v == nil then return nil end
        a[i] = v + 0
    end
    local color = {r = a[1], g = a[2], b = a[3]}
    if #a == 4 then color['a'] = a[4] end
    -- game.print(game.table_to_json(a))
    return color
end

script.on_event(defines.events.on_tick, function(event)
    rendering.clear('ChunkyChunks')
    for _, p in pairs(game.connected_players) do
        if p.is_shortcut_toggled(SHORTCUT_NAME) then
            chunky(p.index, p.mod_settings['chunkychunks-size-1'].value, p.mod_settings['chunkychunks-offset-1'].value, parseColour(p.mod_settings['chunkychunks-color-1'].value))
            chunky(p.index, p.mod_settings['chunkychunks-size-2'].value, p.mod_settings['chunkychunks-offset-2'].value, parseColour(p.mod_settings['chunkychunks-color-2'].value))
        end
    end
end)

script.on_event(defines.events.on_lua_shortcut, toggle)
script.on_event('chunkychunks-toggle', toggle)

script.on_event(defines.events.on_player_toggled_alt_mode, function(event)
    local p = game.players[event.player_index]
    if not p.mod_settings['chunkychunks-follow-alt-mode'].value then return nil end
    enabled(event.player_index, p.game_view_settings.show_entity_info)
end)
