local function has_value(tab, val)
    for index, value in ipairs(tab) do
        if value == val then
            return true
        end
    end
    return false
end

function map(l,f)
    local i = 1
    if type(l) == 'function' then
        return function()
            local q = l()
            if q ~= nil then
                local r = f(q, i, l)
                i = i + 1
                return r
            end
        end
    end
    local r = {}
    for i = 1, #l do
        r[i] = f(l[i], i, l)
    end
    return r
end

function filter(l,f)
    if type(l) == 'function' then
        return function()
            local q = l()
            while q ~= nil do
                if f(q) then return q end
                q = l()
            end
        end
    end
    local r = {}
    for i = 1, #l do
        if f(l[i], i, l) then
            table.insert(r, l[i])
        end
    end
    return r
end

function add(p, q)
    return {p[1] + q[1], p[2] + q[2]}
end

function mult(v, s)
    return {v[1] * s, v[2] * s}
end

function neg(p)
    return {-p[1], -p[2]}
end

function apos(pos)
    return {pos.x, pos.y}
end

function check_for_ghosts(entity)
    local rect_w = 0.1
    local rect_h = 4
    -- game.print(game.table_to_json({entity.get_radius(), entity.supports_direction, entity.direction, defines.direction.east, defines.direction.west}))
    if has_value({defines.direction.east, defines.direction.west}, entity.direction) then
        local swap = rect_w
        rect_w = rect_h
        rect_h = swap
    elseif not has_value({defines.direction.north, defines.direction.south}, entity.direction) then return false end

    local pos = apos(entity.position)
    -- rendering.draw_circle{color=game.players[1].color, width=2, radius=0.5, filled=false, target=pos, time_to_live=600, surface=entity.surface, }
    local area = {add(pos, {-rect_w, -rect_h}), add(pos, {rect_w, rect_h})}
    -- rendering.draw_rectangle{color=game.players[1].color, width=2, filled=false, left_top=area[1], right_bottom=area[2], time_to_live=300, surface=entity.surface, }

    local found_ents = entity.surface.find_entities_filtered{area=area}
    local ghost_train_filter = function(w)
        return w.name == 'entity-ghost' and game.entity_prototypes[w.ghost_type] and game.entity_prototypes[w.ghost_type].weight ~= nil
    end
    local train_filter = function(w)
        return w.name == 'entity-ghost' and game.entity_prototypes[w.ghost_type] and game.entity_prototypes[w.ghost_type].weight ~= nil
    end
    local train = filter(found_ents, train_filter)
    local ghost_train = filter(found_ents, ghost_train_filter)
    -- game.table_to_json(map(train, function(q)
    --     -- rendering.draw_circle{color=game.players[1].color, width=2, radius=0.2, filled=true, target=q.position, time_to_live=600, surface=entity.surface, }
    --     return {q.name, q.ghost_type}
    -- end))
    return #ghost_train == 0
end

-- script.on_event(defines.events.on_robot_built_entity, function(event))
    -- local entity = event.created_entity
    -- game.print(entity.name..' '..entity.type)

script.on_event(defines.events.on_train_created, function(event)
    local train = event.train
    if train == nil then return end
    local front = check_for_ghosts(train.front_rail)
    local back = check_for_ghosts(train.back_rail)
    if front and back then
        train.manual_mode = false
    end
end)