remote.add_interface("LaneBalancer",
{
    clean = function()
        global.LaneBalancer = {}
    end
})

script.on_init(function() OnInit() end)
script.on_configuration_changed(function() OnConfigurationChanged(event) end)
script.on_event(defines.events.on_tick, function(event) OnTick(event) end)
script.on_event(defines.events.on_built_entity, function(event) OnBuiltEntity(event) end)
script.on_event(defines.events.on_entity_died, function(event) OnEntityDied(event) end)
script.on_event(defines.events.on_pre_player_mined_item, function(event) OnPrePlayerMinedItem(event) end)
script.on_event(defines.events.on_player_rotated_entity, function(event) OnPlayerRotatedEntity(event) end)
script.on_event(defines.events.on_robot_built_entity, function(event) OnRobotBuiltEntity(event) end)
script.on_event(defines.events.on_robot_pre_mined, function(event) OnRobotPreMined(event) end)

local north = defines.direction.north
local east = defines.direction.east
local south = defines.direction.south
local west = defines.direction.west


-- add balancer to existing techs
local balancer_techs = {
	["logistics"] = "lane-balancer",
	["logistics-2"] = "fast-lane-balancer",
	["logistics-3"] = "express-lane-balancer",
}


local function EnableRecipes()
	for _, force in pairs(game.forces) do
		for tech, recipe in pairs(balancer_techs) do
			if force.technologies[tech].researched then
				force.recipes[recipe].enabled = true
			end
		end
	end
end 


function OnInit()
    global.LaneBalancer = {	}
	EnableRecipes()
end

function OnConfigurationChanged(_Event)
    for key,_Balancer in pairs(global.LaneBalancer) do 
        if _Balancer.surface == nil then
            _Balancer.surface = game.surfaces[1]
        end 
    end
	EnableRecipes()
end

function OnPlayerRotatedEntity(_Event)
    if IsLaneBalancer(_Event.entity) then
        local _Balancer = _Event.entity
        for key,value in pairs(global.LaneBalancer) do 
            if value.pos.x == _Balancer.position.x and value.pos.y == _Balancer.position.y then
                value.dir = _Balancer.direction
                value.cachedInBelt = nil
                value.cachedOutBelt = nil
                break
            end 
        end
    end
end


local VV = 0.55
local HH = 0.23
local targetOff = {[0] = {left = {x = -HH, y = -VV}, right = {x =  HH, y = -VV}},
    [2] = {left = {x =  VV, y = -HH}, right = {x =  VV, y =  HH}},
    [4] = {left = {x =  HH, y =  VV}, right = {x = -HH, y =  VV}},
    [6] = {left = {x = -VV, y =  HH}, right = {x = -VV, y = -HH}}
}

function MoveItems(ItemName, InputLane, outPos, OutputLane)      
    if OutputLane.can_insert_at_back() then
        local stack = {name = ItemName, count = 1}
        InputLane.remove_item (stack)
		OutputLane.insert_at_back(stack)
        return true
    end
    return false
end

function NegateSide(Side)
    if(Side == 1) then 
        return 2
    else if(Side == 2) then return 1
        else return 1 end
    end
end

function is_right_turn(previous_dir, next_dir)
	return (((previous_dir + 2) % 8) == (next_dir % 8))
end

function is_left_turn(previous_dir, next_dir)
	return ((previous_dir % 8) == ((next_dir + 2) % 8))
end

function is_ns(direction)
  return direction == 0 or direction == 4
end

function is_ew(direction)
  return direction == 2 or direction == 6
end

function opposite_direction(direction)
  if direction >= 4 then
    return direction - 4
  end
  return direction + 4
end

function GetBalancerInBelt(Balancer)  
    local inBelt = Balancer.cachedInBelt			
    if nil == inBelt or not inBelt.valid or (inBelt.type ~= "transport-belt" and inBelt.direction ~= Balancer.dir) then
        Balancer.cachedInBelt = nil
        inBelt = nil
		
        local scanArea = GetScanArea(Balancer.dir, Balancer.pos)
        local belt = Balancer.surface.find_entities_filtered({	type="transport-belt", area = scanArea})
		local underground = Balancer.surface.find_entities_filtered({	type="underground-belt", area = scanArea})
		local splitter = Balancer.surface.find_entities_filtered({	type="splitter", area = scanArea})

		Balancer.laneInOffset = 0

        if belt[1] ~= nil and belt[1].direction == Balancer.dir then			
            inBelt = belt[1]
            Balancer.cachedInBelt = inBelt
		elseif underground[1] ~= nil and underground[1].direction == Balancer.dir and underground[1].belt_to_ground_type == "output" then
			inBelt = underground[1]
            Balancer.cachedInBelt = inBelt
		elseif splitter[1] ~= nil and splitter[1].direction == Balancer.dir then		
			scanArea = GetScanArea(Balancer.dir, OffsetScanPos(Balancer.dir, Balancer.pos, -1.0))
			local leftSplitter = Balancer.surface.find_entities_filtered({	type="splitter", area = scanArea})
			scanArea = GetScanArea(Balancer.dir, OffsetScanPos(Balancer.dir, Balancer.pos, 1.0))
			local rightSplitter = Balancer.surface.find_entities_filtered({	type="splitter", area = scanArea})

			if leftSplitter[1] ~= nil and leftSplitter[1] == splitter[1] then
				Balancer.laneInOffset = 6
				inBelt = splitter[1]
                Balancer.cachedInBelt = inBelt
			elseif rightSplitter[1] ~= nil and rightSplitter[1] == splitter[1] then
				Balancer.laneInOffset = 4
				inBelt = splitter[1]
                Balancer.cachedInBelt = inBelt
			end

        end
    end
    return inBelt
end

function GetBalancerOutBelt(Balancer)  
    local outBelt = Balancer.cachedOutBelt			
    if nil == outBelt or not outBelt.valid or (outBelt.type ~= "transport-belt" and outBelt.direction ~= Balancer.dir) then
        Balancer.cachedOutBelt = nil
        outBelt = nil
		
        local facing_directions = {[north] = south, [east] = west, [south] = north, [west] = east}
        local scanArea = GetScanArea(facing_directions[Balancer.dir], Balancer.pos)
    
        local belt = Balancer.surface.find_entities_filtered({	type="transport-belt", area = scanArea})
		local underground = Balancer.surface.find_entities_filtered({	type="underground-belt", area = scanArea})
		local splitter = Balancer.surface.find_entities_filtered({	type="splitter", area = scanArea})

		Balancer.laneOutForce = 0
		Balancer.laneOutMult = 1
		Balancer.laneOutOffset = 0


        if belt[1] ~= nil then			
			if is_right_turn(Balancer.dir, belt[1].direction) then
				Balancer.laneOutMult = 0
				Balancer.laneOutForce = 2
			elseif is_left_turn(Balancer.dir, belt[1].direction) then
				Balancer.laneOutMult = 0
				Balancer.laneOutForce = 1
			end
			outBelt = belt[1]
            Balancer.cachedOutBelt = outBelt
		elseif underground[1] ~= nil and underground[1].direction == Balancer.dir and underground[1].belt_to_ground_type == "input" then
			outBelt = underground[1]
            Balancer.cachedOutBelt = outBelt
		elseif splitter[1] ~= nil and splitter[1].direction == Balancer.dir then
			scanArea = GetScanArea(facing_directions[Balancer.dir], OffsetScanPos(Balancer.dir, Balancer.pos, -1.0))
			local leftSplitter = Balancer.surface.find_entities_filtered({	type="splitter", area = scanArea})
			scanArea = GetScanArea(facing_directions[Balancer.dir], OffsetScanPos(Balancer.dir, Balancer.pos, 1.0))
			local rightSplitter = Balancer.surface.find_entities_filtered({	type="splitter", area = scanArea})

			if leftSplitter[1] ~= nil and leftSplitter[1] == splitter[1] then
				Balancer.laneOutOffset = 2
				outBelt = splitter[1]
                Balancer.cachedOutBelt = outBelt
			elseif rightSplitter[1] ~= nil and rightSplitter[1] == splitter[1] then
				Balancer.laneOutOffset = 0
				outBelt = splitter[1]
                Balancer.cachedOutBelt = outBelt
			end
        end
    end
    return outBelt
end

function OnTick(_Event)	  
	
	local tickSlow = (game.tick % 4) == 0
	local tickExtraSlow = (game.tick % 8) == 0

    for _, balancer in pairs(global.LaneBalancer) do

		if balancer.balancerSpeed == 3 or (balancer.balancerSpeed == 2 and tickSlow) or tickExtraSlow then
		
            local inBelt = GetBalancerInBelt(balancer)
            local outBelt = GetBalancerOutBelt(balancer)

            if (balancer.lastOut == nil) then balancer.lastOut = {1 , 1} end
            
            if nil ~= inBelt and nil ~= outBelt then
                for i=1,2 do		
                    local takeFromSide = NegateSide(balancer.lastInSide)
                    local takeToSide = NegateSide(balancer.lastOut[takeFromSide])	
                    
                    
                    local InLane = inBelt.get_transport_line(takeFromSide + balancer.laneInOffset)
                    local OutLane = outBelt.get_transport_line(takeToSide * balancer.laneOutMult + balancer.laneOutForce + balancer.laneOutOffset)
                                                    
                    if #InLane == 0 then
                        takeFromSide = NegateSide(takeFromSide)
                        takeToSide = NegateSide(balancer.lastOut[takeFromSide])
                        InLane = inBelt.get_transport_line(takeFromSide + balancer.laneInOffset)
                        OutLane = outBelt.get_transport_line(takeToSide * balancer.laneOutMult + balancer.laneOutForce + balancer.laneOutOffset)					
                    end	
    
                    if #InLane > 0 then
                        local InLaneContents=InLane.get_contents()	
                        for itemName,_  in pairs(InLaneContents) do 
                        
                            local moved = true
                            if not MoveItems(itemName, InLane, outPos, OutLane) then 
                                takeToSide = NegateSide(takeToSide)	
                                OutLane = outBelt.get_transport_line(takeToSide * balancer.laneOutMult + balancer.laneOutForce + balancer.laneOutOffset)
                                if not MoveItems(itemName, InLane, outPos, OutLane) then
                                    moved = false
                                end
                            end	
                            
                            if (moved) then 
                                balancer.lastInSide = takeFromSide
                                balancer.lastOut[takeFromSide] = takeToSide
                            end
                        end												
                    end	
                end
            end
        end
    end
end
				   
function OnBuiltEntity(_Event)
    if IsLaneBalancer(_Event.created_entity) then
        AddLaneBalancer(_Event.created_entity)
    end
end

function OnRobotBuiltEntity(_Event)
    if IsLaneBalancer(_Event.created_entity) then
        AddLaneBalancer(_Event.created_entity)
    end
end

function OnRobotPreMined(_Event)
    if IsLaneBalancer(_Event.entity) then
        RemoveLaneBalancer(_Event.entity)
    end
end

function OnPrePlayerMinedItem(_Event)
    if IsLaneBalancer(_Event.entity) then
        RemoveLaneBalancer(_Event.entity)
    end
end

function OnEntityDied(_Event)
    if IsLaneBalancer(_Event.entity) then
        RemoveLaneBalancer(_Event.entity)
    end
end

function IsLaneBalancer(_Entity)
    return (_Entity.name == "lane-balancer" or _Entity.name == "fast-lane-balancer" or _Entity.name == "express-lane-balancer")
end

function AddSet(_Set)
    table.insert(global.LaneBalancer, _Set)
end

function AddLaneBalancer(_Balancer)
    local _Position = _Balancer.position
    local _Direction = _Balancer.direction
	local _Surface = _Balancer.surface
	local _BalancerSpeed = 1
	if _Balancer.name == "fast-lane-balancer" then _BalancerSpeed = 2 
	elseif _Balancer.name == "express-lane-balancer" then _BalancerSpeed = 3 
	end

    local set = {pos = _Position, dir = _Direction, surface = _Surface, lastInSide = 1, lastOut = {1,2}, laneInOffset = 0, laneOutMult = 1, laneOutForce = 0, laneOutOffset = 0, balancerSpeed = _BalancerSpeed}		
    AddSet(set)	
end

function RemoveLaneBalancer(_Balancer)		
    for key,balancerInDB in pairs(global.LaneBalancer) do 
        if balancerInDB.pos.x == _Balancer.position.x and balancerInDB.pos.y == _Balancer.position.y and _Balancer.surface == balancerInDB.surface then
			table.remove(global.LaneBalancer, key)
			break         
        end 
    end
end

function OffsetScanPos(_Direction, _Position, offsetDir )
	local outPos = _Position
	if _Direction == north then
		outPos.x = outPos.x + offsetDir
		return outPos
    elseif _Direction == east then
        outPos.y = outPos.y + offsetDir
		return outPos
    elseif _Direction == south then
        outPos.x = outPos.x - offsetDir
		return outPos
    elseif _Direction == west then
        outPos.y = outPos.y - offsetDir
		return outPos
    else
        return _Position
    end

end

function GetScanArea(_Direction, _Position )
    local beltscan_coords = { -- Points to search for transport belts.
        [north] = {{_Position.x - 0.3, _Position.y - 1.1},{_Position.x + 0.3, _Position.y - 0.8}},
        [east] =  {{_Position.x + 1.1  , _Position.y - 0.3},{_Position.x + 0.8, _Position.y + 0.3}},
        [south] = {{_Position.x - 0.3, _Position.y + 1.1},{_Position.x + 0.3, _Position.y + 0.8}},
        [west] =  {{_Position.x - 1.1  , _Position.y - 0.3},{_Position.x - 0.8, _Position.y + 0.3}}
    }

    if _Direction == north then
        return beltscan_coords[south]
    elseif _Direction == east then
        return beltscan_coords[west]
    elseif _Direction == south then
        return beltscan_coords[north]
    elseif _Direction == west then
        return beltscan_coords[east]
    else
        return {{pos.x , pos.y},{pos.x , pos.y}}
    end
end

